@@ -11,7 +11,7 @@ NULL
 get_idd_group_index <- function (idd_env, group = NULL) {
     if (is.null(group)) return(idd_env$group$group_id)
 
-    assert(is.character(group))
+    assert(are_string(group))
 
     res <- idd_env$group[J(group), on = "group_name", group_id]
     if (anyNA(res)) abort_bad_key("error_group_name", "group name", group)
@@ -49,7 +49,20 @@ get_idd_class <- function (idd_env, class = NULL, property = NULL, underscore =
         if (is.null(property)) {
             return(idd_env$class[, .SD, .SDcols = cols])
         } else {
-            return(idd_env$class[, .SD, .SDcols = unique(c(cols, property))])
+            if ("group_name" %chin% property) {
+                property <- setdiff(property, "group_name")
+                add_group <- TRUE
+            } else {
+                add_group <- FALSE
+            }
+
+            res <- idd_env$class[, .SD, .SDcols = unique(c(cols, property))]
+
+            if (add_group) {
+                add_joined_cols(idd_env$group, res, "group_id", "group_name")
+            }
+
+            return(res)
         }
     }
 
@@ -116,7 +129,7 @@ get_idd_class_field_num <- function (dt_class, num = NULL) {
         ]
     }
 
-    if (nrow(dt_class[input_num > num_fields])) {
+    if (any(dt_class$input_num > dt_class$num_fields)) {
         dt_class[input_num > num_fields,
             acceptable_num := pmax(num_fields, min_fields, last_required, last_extensible)]
         dt_class[input_num <= num_fields,
@@ -304,6 +317,7 @@ get_idd_field_from_which <- function (idd_env, class, field, underscore = TRUE,
             col_on <- "field_name"
             set(dt_in, NULL, "field_name", field)
         }
+        setindexv(dt_in, c("class_id", col_on))
 
         # join
         dt_join <- idd_env$field[dt_in, on = c("class_id", col_on), allow.cartesian = TRUE]
